﻿namespace HeuristicLab.BenchmarkSuite.Problems {
  using System.Linq;

  using Common;
  using Core;
  using Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  public class Data : NamedItem {
    public Data(double bestResult, double worstResult, int testCount, int trainingCount,
      int inputArgumentCount, int outputArgumentCount, Example[] exmaples, IExampleParser exampleParser) {
      WorstResult = worstResult;
      BestResult = bestResult;
      OriginalTrainingCount = trainingCount;
      OriginalTestCount = testCount;
      InputArgumentCount = inputArgumentCount;
      OutputArgumentCount = outputArgumentCount;
      Examples = exmaples;
      ExampleParser = exampleParser;

      Description = "Examples";
    }

    [StorableConstructor]
    public Data(bool deserializing) : base(deserializing) { }

    public Data(Data origin, Cloner cloner) : base(origin, cloner) {
      this.BestResult = origin.BestResult;
      this.WorstResult = origin.WorstResult;
      this.OriginalTestCount = origin.OriginalTestCount;
      this.OriginalTrainingCount = origin.OriginalTrainingCount;
      this.Examples = origin.Examples.Select(cloner.Clone).ToArray();
    }

    [Storable]
    public readonly IExampleParser ExampleParser;
    [Storable]
    public readonly Example[] Examples;
    [Storable]
    public readonly int OriginalTestCount;
    [Storable]
    public readonly int OriginalTrainingCount;
    [Storable]
    public readonly double WorstResult;
    [Storable]
    public readonly double BestResult;
    [Storable]
    public readonly int InputArgumentCount;
    [Storable]
    public readonly int OutputArgumentCount;

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Data(this, cloner);
    }
  }
}
